
// -------------------------------------------------------------------
// Copyright (C) 2012 by Sascha Wilde <wilde@sha-bang.de>

// This program is free software under the GNU GPL (>=v2)
// Read the file COPYING coming with the software for details.
// -------------------------------------------------------------------

package de.sha_bang.OrderApp;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.os.Bundle;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.lang.ClassNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.Toast;

public class OrderApp extends Activity
{
    public final static String EXTRA_ORDERLIST = "de.sha-bang.OrderApp.ORDERLIST";

    private static final int DIALOG_NEW_ITEM = 0;
    private static final int DIALOG_TAP_TO_DEL = 1;

    private SimpleAdapter myOrder;
    private ArrayList<HashMap<String, String>> orderList;
    private boolean deleteMode = false;

    private void initOrderList() {
        orderList = new ArrayList<HashMap<String, String>>();
        HashMap<String, String> map = new HashMap<String, String>();
        map.put("item", "Glühwein");
        map.put("count", "0");
        map.put("total", "0");
        orderList.add(map);
        map = new HashMap<String, String>();
        map.put("item", "Glühwein Schlehen");
        map.put("count", "0");
        map.put("total", "0");
        orderList.add(map);
        map = new HashMap<String, String>();
        map.put("item", "Glühwein Holunder");
        map.put("count", "0");
        map.put("total", "0");
        orderList.add(map);
    }

    private void updateOrder(AdapterView<?> parent, int position, int num) {
        HashMap<String, String> entry = 
            (HashMap<String, String>) parent.getItemAtPosition (position);
        int count = Integer.parseInt(entry.get("count"));
        if (num == 0) {
            // special operation: delete item
            orderList.remove(entry);
            myOrder.notifyDataSetChanged();
        } else if (num > 0 || count > 0) {
            entry.put("count", "" + (count + num)); 
            myOrder.notifyDataSetChanged();
            Toast.makeText(getApplicationContext(), 
                           String.format(((num > 0) ? getResources().getText(R.string.toast_ack_more).toString() : 
                                          getResources().getText(R.string.toast_ack_less).toString()), 
                                         entry.get("item")),
                           Toast.LENGTH_SHORT).show();
        }
    }

    private void resetOrders() {
        for(int i=0;i<orderList.size();i++) {
            orderList.get(i).put("count", "0");
        }
        myOrder.notifyDataSetChanged();
        Toast.makeText(getApplicationContext(), 
                       R.string.toast_reset,
                       Toast.LENGTH_SHORT).show();
    }

    private void totalAdd() {
        int count, total;
        for(int i=0;i<orderList.size();i++) {
            count = Integer.parseInt(orderList.get(i).get("count"));
            total = Integer.parseInt(orderList.get(i).get("total"));
            orderList.get(i).put("total", "" + (total + count));
            orderList.get(i).put("count", "0");
        }
        myOrder.notifyDataSetChanged();
        Toast.makeText(getApplicationContext(),
                       R.string.toast_totaladd,
                       Toast.LENGTH_SHORT).show();
    }

    private void totalShow() {
        Bundle extraBundle = new Bundle(1);
        Intent intent = new Intent(this, ShowTotalActivity.class);
        extraBundle.putSerializable("orderList", orderList);
        intent.putExtra(EXTRA_ORDERLIST, extraBundle);
        startActivity(intent);
    }

    private void totalReset() {
        for(int i=0;i<orderList.size();i++) {
            orderList.get(i).put("total", "0");
        }
        Toast.makeText(getApplicationContext(), 
                       R.string.toast_totalreset,
                       Toast.LENGTH_SHORT).show();
    }

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        ListView list = (ListView) findViewById(R.id.ORDERS);

        if (savedInstanceState != null && savedInstanceState.containsKey("orderList")) {
            orderList = 
                (ArrayList<HashMap<String, String>>) savedInstanceState.getSerializable("orderList");
        } else {
            try {
                FileInputStream fis = openFileInput("orderlist.data");
                ObjectInputStream ois = new ObjectInputStream(fis);
                orderList = (ArrayList<HashMap<String, String>>) ois.readObject();
                ois.close();
                fis.close();
            } catch (FileNotFoundException e) {
                initOrderList();
            } catch (ClassNotFoundException e) {
                initOrderList();
            } catch (IOException e) {
                System.out.println(e);
                initOrderList();
            }
        }

        myOrder = 
            new SimpleAdapter(this, orderList, R.layout.order_row,
                              new String[] {"item", "count"},
                              new int[] {R.id.ITEM_CELL, R.id.COUNT_CELL});
        list.setAdapter(myOrder);

        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                public void onItemClick(AdapterView<?> parent, View view,
                                        int position, long id) {
                    // When clicked, add one item unless in delete mode...
                    if (!deleteMode) {
                        updateOrder(parent, position, 1);
                    } else {
                        // in delete mode delete item
                        updateOrder(parent, position, 0);
                        deleteMode = false;
                    }
                }
            });

        list.setOnItemLongClickListener(new AdapterView.OnItemLongClickListener() {
                public boolean onItemLongClick(AdapterView<?> parent, View view,
                                               int position, long id) {
                    // When long clicked, remove one item...
                    updateOrder(parent, position, -1);
                    deleteMode = false;
                    return true;
                }
            });
    }

    @Override
    public void onSaveInstanceState (Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putSerializable("orderList", orderList);
    }

    @Override
    public void onPause () {
        super.onPause();
        try {
            FileOutputStream fos = openFileOutput("orderlist.data", Context.MODE_PRIVATE);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeObject(orderList);
            oos.close();
            fos.close();
        } catch (IOException e) {
            System.out.println(e);
        }
    }

    @Override
    protected Dialog onCreateDialog(int id) {
        switch (id) {
        case DIALOG_NEW_ITEM:
            LayoutInflater factory = LayoutInflater.from(this);
            final View textEntryView = factory.inflate(R.layout.alert_dialog_text_entry, null);
            AlertDialog inputDialog;
             
            return new AlertDialog.Builder(this)
                .setTitle(R.string.dialog_title_new_item)
                .setView(textEntryView)
                .setPositiveButton(R.string.dialog_ok, new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int whichButton) {
                            /* User clicked OK so add Item */
                            HashMap<String, String> map = new HashMap<String, String>();
                            map.put("item",
                                    ((EditText) textEntryView.findViewById(R.id.itemname_edit))
                                    .getText().toString());
                            map.put("count", "0");
                            map.put("total", "0");
                            orderList.add(map);
                            myOrder.notifyDataSetChanged();        
                        }
                    })
                .setNegativeButton(R.string.dialog_cancel, new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int whichButton) {
                            /* User clicked cancel so do nothing */
                        }
                    })
                .create();
        case DIALOG_TAP_TO_DEL:
            return new AlertDialog.Builder(this)
                .setIcon(android.R.drawable.ic_dialog_info)
                .setTitle(R.string.dialog_title_tap_to_delete)
                .setPositiveButton(R.string.dialog_ok, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int whichButton) {
                        deleteMode = true;
                    }
                })
                .setNegativeButton(R.string.dialog_cancel, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int whichButton) {
                        deleteMode = false;
                    }
                })
                .create();
        }
        return null;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case R.id.add:
            deleteMode = false;
            showDialog(DIALOG_NEW_ITEM);
            return true;
        case R.id.del:
            showDialog(DIALOG_TAP_TO_DEL);
            return true;
        case R.id.reset:
            deleteMode = false;
            resetOrders();
            return true;
        case R.id.totaladd:
            deleteMode = false;
            totalAdd();
            return true;
        case R.id.totalshow:
            deleteMode = false;
            totalShow();
            return true;
        case R.id.totalreset:
            deleteMode = false;
            totalReset();
            return true;
        default:
            return super.onOptionsItemSelected(item);
        }
    }

}
